local BD = LibStub("ButtonDaemon-1.0")
DemonDaemon = LibStub("AceAddon-3.0"):NewAddon("DemonDaemon", "AceEvent-3.0")
local AceDB = LibStub("AceDB-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local dialog = LibStub("AceConfigDialog-3.0")

local options = {
	type = 'group', childGroups = "tab",
	args = {
		main = {
			type = 'group', name ='Main Buttons', order = 1, 
			args = {
				radius = {
					type = 'range', name = "Radius",
					desc = "Set the Radius for placing the main buttons",
					handler = DemonDaemon,
					min = 15, max = 1000,
					step = 1,
					--pattern = "%d",
					get = "getRadius",
					set = "setRadius",
					order = 1,
				},
				angle = {
					type = 'range', name = "Start Angle",
					desc = "Set the starting Angle for placing the main buttons",
					handler = DemonDaemon,
					min = 0, max = 359,
					step = 1,
					--pattern = "%d",
					get = "getAngle",
					set = "setAngle",
					order = 2,
				},
				override = {
					type = 'toggle', name = "Custom Increment",
					desc = "Toggle over-riding increment angle",
					handler = DemonDaemon,
					get = "getOverrideState",
					set = "setOverrideState",
					order = 3,
				},
				increment = {
					type = 'range', name = "Increment Angle",
					desc = "Set the Increment Angle for placing the main buttons",
					handler = DemonDaemon,
					min = -359, max = 359,
					step = 1,
					--pattern = "%d",
					get = "getIncrement",
					set = "setIncrement",
					order = 4,
				},
				reverse = {
					type = 'toggle', name = "Reverse Placement",
					desc = "Toggle direction of button placement. Does not apply to custom Increment Angles",
					handler = DemonDaemon,
					get = "getReverseState",
					set = "setReverseState",
					order = 5,
				},
				anchor = {
					type = 'input', name = "Anchor Frame",
					desc = "Set the Frame to place the main buttons around. Defaults to Minimap",
					handler = DemonDaemon,
					--pattern = "%d",
					get = "getAnchor",
					set = "setAnchor",
					order = 6,
				},
				straight = {
					type = 'toggle', name = "Toggle Alignment",
					desc = "Toggle alignment of main buttons between circular and straight",
					handler = DemonDaemon,
					get = "getStraightState",
					set = "setStraightState",
					order = 7,
				},
				offx = {
					type = 'range', name = "X Offset",
					desc = "Set the X Offset for Placing the main buttons",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "getMainOffX",
					set = "setMainOffX",
					order = 8,
				},
				offy = {
					type = 'range', name = "Y Offset",
					desc = "Set the Y Offset for Placing the main buttons",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "getMainOffY",
					set = "setMainOffY",
					order = 9,
				},
			},
		},
		firstrow = {
			type = 'group', name ='1st Row', order = 2,
			args = {
				initx = {
					type = 'range', name = "Initial X",
					desc = "Set the Initial X Posistion for Placing the first row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get1InitX",
					set = "set1InitX",
					order = 1,
				},
				inity = {
					type = 'range', name = "Initial Y",
					desc = "Set the Initial Y Posistion for Placing the first row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get1InitY",
					set = "set1InitY",
					order = 2,
				},
				offx = {
					type = 'range', name = "X Offset",
					desc = "Set the X Offset for Placing the first row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get1OffX",
					set = "set1OffX",
					order = 3,
				},
				offy = {
					type = 'range', name = "Y Offset",
					desc = "Set the Y Offset for Placing the first row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get1OffY",
					set = "set1OffY",
					order = 4,
				},
			},
		},
		secondrow = {
			type = 'group', name ='2nd Row', order = 3,
			args = {
				initx = {
					type = 'range', name = "Initial X",
					desc = "Set the Initial X Posistion for Placing the second row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get2InitX",
					set = "set2InitX",
					order = 1,
				},
				inity = {
					type = 'range', name = "Initial Y",
					desc = "Set the Initial Y Posistion for Placing the second row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get2InitY",
					set = "set2InitY",
					order = 2,
				},
				offx = {
					type = 'range', name = "X Offset",
					desc = "Set the X Offset for Placing the second row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get2OffX",
					set = "set2OffX",
					order = 3,
				},
				offy = {
					type = 'range', name = "Y Offset",
					desc = "Set the Y Offset for Placing the second row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get2OffY",
					set = "set2OffY",
					order = 4,
				},
			},
		},
		thirdrow = {
			type = 'group', name ='3rd Row', order = 4,
			args = {
				initx = {
					type = 'range', name = "Initial X",
					desc = "Set the Initial X Posistion for Placing the third row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get3InitX",
					set = "set3InitX",
					order = 1,
				},
				inity = {
					type = 'range', name = "Initial Y",
					desc = "Set the Initial Y Posistion for Placing the third row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get3InitY",
					set = "set3InitY",
					order = 2,
				},
				offx = {
					type = 'range', name = "X Offset",
					desc = "Set the X Offset for Placing the third row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get3OffX",
					set = "set3OffX",
					order = 3,
				},
				offy = {
					type = 'range', name = "Y Offset",
					desc = "Set the Y Offset for Placing the third row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get3OffY",
					set = "set3OffY",
					order = 4,
				},
			},
		},
		fourthrow = {
			type = 'group', name ='4th Row', order = 5,
			args = {
				initx = {
					type = 'range', name = "Initial X",
					desc = "Set the Initial X Posistion for Placing the forth row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get4InitX",
					set = "set4InitX",
					order = 1,
				},
				inity = {
					type = 'range', name = "Initial Y",
					desc = "Set the Initial Y Posistion for Placing the forth row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get4InitY",
					set = "set4InitY",
					order = 2,
				},
				offx = {
					type = 'range', name = "X Offset",
					desc = "Set the X Offset for Placing the forth row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get4OffX",
					set = "set4OffX",
					order = 3,
				},
				offy = {
					type = 'range', name = "Y Offset",
					desc = "Set the Y Offset for Placing the forth row",
					handler = DemonDaemon,
					min = -100, max = 100,
					step = 1,
					--pattern = "%d",
					get = "get4OffY",
					set = "set4OffY",
					order = 4,
				},
			},
		},
	}
}

function DemonDaemon:get1InitX()
	return self.db.profile.oneinitx
end
function DemonDaemon:set1InitX(info, s)
	self.db.profile.oneinitx = s
	self:PosistionButtons()
end
function DemonDaemon:get1InitY()
	return self.db.profile.oneinity
end
function DemonDaemon:set1InitY(info, s)
	self.db.profile.oneinity = s
	self:PosistionButtons()
end
function DemonDaemon:get1OffX()
	return self.db.profile.oneoffx
end
function DemonDaemon:set1OffX(info, s)
	self.db.profile.oneoffx = s
	self:PosistionButtons()
end
function DemonDaemon:get1OffY()
	return self.db.profile.oneoffy
end
function DemonDaemon:set1OffY(info, s)
	self.db.profile.oneoffy = s
	self:PosistionButtons()
end

function DemonDaemon:get2InitX()
	return self.db.profile.twoinitx
end
function DemonDaemon:set2InitX(info, s)
	self.db.profile.twoinitx = s
	self:PosistionButtons()
end
function DemonDaemon:get2InitY()
	return self.db.profile.twoinity
end
function DemonDaemon:set2InitY(info, s)
	self.db.profile.twoinity = s
	self:PosistionButtons()
end
function DemonDaemon:get2OffX()
	return self.db.profile.twooffx
end
function DemonDaemon:set2OffX(info, s)
	self.db.profile.twooffx = s
	self:PosistionButtons()
end
function DemonDaemon:get2OffY()
	return self.db.profile.twooffy
end
function DemonDaemon:set2OffY(info, s)
	self.db.profile.twooffy = s
	self:PosistionButtons()
end

function DemonDaemon:get3InitX()
	return self.db.profile.threeinitx
end
function DemonDaemon:set3InitX(info, s)
	self.db.profile.threeinitx = s
	self:PosistionButtons()
end
function DemonDaemon:get3InitY()
	return self.db.profile.threeinity
end
function DemonDaemon:set3InitY(info, s)
	self.db.profile.threeinity = s
	self:PosistionButtons()
end
function DemonDaemon:get3OffX()
	return self.db.profile.threeoffx
end
function DemonDaemon:set3OffX(info, s)
	self.db.profile.threeoffx = s
	self:PosistionButtons()
end
function DemonDaemon:get3OffY()
	return self.db.profile.threeoffy
end
function DemonDaemon:set3OffY(info, s)
	self.db.profile.threeoffy = s
	self:PosistionButtons()
end

function DemonDaemon:get4InitX()
	return self.db.profile.fourinitx
end
function DemonDaemon:set4InitX(info, s)
	self.db.profile.fourinitx = s
	self:PosistionButtons()
end
function DemonDaemon:get4InitY()
	return self.db.profile.fourinity
end
function DemonDaemon:set4InitY(info, s)
	self.db.profile.fourinity = s
	self:PosistionButtons()
end
function DemonDaemon:get4OffX()
	return self.db.profile.fouroffx
end
function DemonDaemon:set4OffX(info, s)
	self.db.profile.fouroffx = s
	self:PosistionButtons()
end
function DemonDaemon:get4OffY()
	return self.db.profile.fouroffy
end
function DemonDaemon:set4OffY(info, s)
	self.db.profile.fouroffy = s
	self:PosistionButtons()
end

function DemonDaemon:getMainOffX()
	return self.db.profile.mainoffx
end
function DemonDaemon:setMainOffX(info, s)
	self.db.profile.mainoffx = s
	self:PosistionButtons()
end
function DemonDaemon:getMainOffY()
	return self.db.profile.mainoffy
end
function DemonDaemon:setMainOffY(info, s)
	self.db.profile.mainoffy = s
	self:PosistionButtons()
end
function DemonDaemon:getStraightState()
	return self.db.profile.straight
end
function DemonDaemon:setStraightState()
	self.db.profile.straight = not self.db.profile.straight
	self:PosistionButtons()
end
function DemonDaemon:getRadius()
	return self.db.profile.radius
end
function DemonDaemon:setRadius(info, s)
	self.db.profile.radius = s
	self:PosistionButtons()
end
function DemonDaemon:getAngle()
	return self.db.profile.angle
end
function DemonDaemon:setAngle(info, s)
	self.db.profile.angle = s
	self:PosistionButtons()
end
function DemonDaemon:getReverseState()
	if self.db.profile.reverse == -1 then
		return true
	else
		return false
	end
end
function DemonDaemon:setReverseState()
	self.db.profile.reverse = self.db.profile.reverse * -1
	self:PosistionButtons()
end
function DemonDaemon:getOverrideState()
	return self.db.profile.override
end
function DemonDaemon:setOverrideState()
	self.db.profile.override = not self.db.profile.override
	self:PosistionButtons()
end
function DemonDaemon:getIncrement()
	return self.db.profile.increment
end
function DemonDaemon:setIncrement(info, s)
	self.db.profile.increment = s
	self:PosistionButtons()
end
function DemonDaemon:getAnchor()
	return self.db.profile.anchor
end
function DemonDaemon:setAnchor(info, s)
	self.db.profile.anchor = s
	if strlower(s) == "cursor" then
		self.db.profile.curx, self.db.profile.cury = GetCursorPosition()
		self.anchor:SetPoint("CENTER",UIParent,"BOTTOMLEFT",self.db.profile.curx / UIParent:GetEffectiveScale(), self.db.profile.cury / UIParent:GetEffectiveScale() )
	else
		self.anchor:SetPoint("CENTER",UIParent,"BOTTOMLEFT",_G[s]:GetCenter() )
	end
	self:PosistionButtons()
end


local function print(mess)
 DEFAULT_CHAT_FRAME:AddMessage(mess)
end

DemonDaemon.SoulShard = 6265
DemonDaemon.HealthstoneID = {
	[5512] = 1,
	[19004] = 1,
	[19005] = 1,
	[5511] = 1,
	[19006] = 1,
	[19007] = 1,
	[5509] = 1,
	[19008] = 1,
	[19009] = 1,
	[5510] = 1,
	[19010] = 1,
	[19011] = 1,
	[9421] = 1,
	[19012] = 1,
	[19013] = 1,
	[22103] = 1, --Master Healthstones
	[22104] = 1,
	[22105] = 1,
	[36889] = 1,
	[36890] = 1,
	[36891] = 1,
	[36892] = 1,
	[36893] = 1,
	[36894] = 1,
}
DemonDaemon.SoulstoneID = {
	[36895] = 1,
	[22116] = 1,
	[16896] = 1,
	[16895] = 1,
	[16893] = 1,
	[16892] = 1,
	[5232] = 1,
	
}
DemonDaemon.SpellstoneID = {
	-- [36897] = 1,
	-- [36896] = 1,
	-- [22646] = 1, --Master Spellstone
	-- [13603] = 1,
	-- [13602] = 1,
	-- [5522] = 1,
	
	[41191]=1,
	[41192]=1,
	[41193]=1,
	[41194]=1,
	[41195]=1,
	[41196]=1,
}
DemonDaemon.FirestoneID = {
	-- [22128] = 1, --Master Firestone
	-- [13701] = 1,
	-- [13700] = 1,
	-- [13699] = 1,
	-- [1254] = 1,
	
	[41170]=1,
	[41169]=1,
	[41171]=1,
	[41172]=1,
	[40773]=1,
	[41173]=1,
	[41174]=1,
}

DemonDaemon.spellID = {
	CreateSS = {
		[1] = 693,
		[2] = 20752,
		[3] = 20755,
		[4] = 20756,
		[5] = 20757,
	},
	
	CreateSP = {
		[1] = 2362,
		[2] = 17727,
		[3] = 17728,
	},
	
	CreateFS = {
		[1] = 6366,
		[2] = 17951,
		[3] = 17952,
		[4] = 17953,
	},
	
	CreateHS = {
		[1] = 6201,
		[2] = 6202,
		[3] = 5699,
		[4] = 11729,
		[5] = 11730,
	},
	
	FelDomination = 18708,
	Enslave = 1098,
	Sacrifice = 18788,
	SoulLink = 19028,
	Empower = 47193,
	
	Imp = 688,
	Voidwalker = 697,
	Succubus = 712,
	Felhunter = 691,
	Infernal = 1122,
	Doomguard = 18540,
	Felguard = 30146,
	
	DetectInvis = 132,
	UnendingBreath = 5697,
	SenseDemons = 5500,
	DemonSkin = 687,
	DemonArmor = 706,
	FelArmor = 28176,
	ShadowWard = 6229,
	Metamorph = 47241,
	
	Felsteed = 5784,
	Dreadsteed = 23161,
	Summon = 698,
	Eye = 126,
	Banish = 710,
	SoulWell = 29893,
	SummonCircle = 48018,
	PortCircle = 48020,
	
	UseHealthstone = {
		[23469] = 1,
		[23468] = 1,
		[6262] = 1,
		[11732] = 1,
		[23476] = 1,
		[23477] = 1,
		[23470] = 1,
		[23471] = 1,
		[6263] = 1,
		[23473] = 1,
		[5720] = 1,
		[23472] = 1,
		[5723] = 1,
		[23474] = 1,
		[23475] = 1,
	},
	UseSoulstone = {
		[20762] = 1,
		[20765] = 1,
		[20764] = 1,
		[20763] = 1,
		[20707] = 1,
	}
	
}
local CreateButtonsFlag, SetupButtonsFlag, PosistionButtonsFlag

function DemonDaemon:OnInitialize()
	self.db = AceDB:New("DemonDaemonDB",{
		profile = {
			radius = 105,
			angle = 270,
			reverse = 1,
			increment = 15,
			mainoffx = 28,
			mainoffy = 0,
			oneinitx = -28,
			oneinity = 28,
			oneoffx = -28,
			oneoffy = 0,
			twoinitx = -28,
			twoinity = 0,
			twooffx = -28,
			twooffy = 0,
			threeinitx = -28,
			threeinity = -28,
			threeoffx = -28,
			threeoffy = 0,
			fourinitx = -28,
			fourinity = -56,
			fouroffx = -28,
			fouroffy = 0,
		}
	},"profile")
	AceConfig:RegisterOptionsTable("DemonDaemon", options)
	dialog:AddToBlizOptions("DemonDaemon", "DemonDaemon")
end

function DemonDaemon:OnEnable()
	self:CreateButtons()
	self:SPELLS_CHANGED()
	
	self.buttons[1]:Execute( [[MenuButtons = newtable(self:GetChildren())]])
	for _, button in ipairs({self.buttons[1]:GetChildren()}) do
		self.buttons[1]:WrapScript(button, "OnClick",
			[[
			if button == "LeftButton" then
				return button, "hide"
			else
				return button, nil
			end
			]],
			[[for i, button in ipairs(MenuButtons) do
				button:Hide()
			end]]
		)
		button:SetScript("OnEnter", function(self) GameTooltip_SetDefaultAnchor(GameTooltip, UIParent); GameTooltip:SetHyperlink("spell:"..select(7,GetSpellInfo(self:GetAttribute("spell"),"")) or "23161") end)
		button:SetScript("OnLeave", function(self) GameTooltip:Hide() end )
	end
	for i=2, 5, 1 do
			self.buttons[i]:SetScript("OnEnter", function(self) GameTooltip_SetDefaultAnchor(GameTooltip, UIParent);  if self:GetAttribute("spell2") then GameTooltip:SetHyperlink("spell:"..select(7,GetSpellInfo(self:GetAttribute("spell2"),"")) or "23161") end end)
			self.buttons[i]:SetScript("OnLeave", function(self) GameTooltip:Hide() end )
	end
	self:RegisterEvent("LEARNED_SPELL_IN_TAB", "SPELLS_CHANGED")
	self:RegisterEvent("SPELLS_CHANGED", "SPELLS_CHANGED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("BAG_UPDATE")
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	
	
	soulTime = self.buttons[2]:CreateFontString("DDSoulTime", "OVERLAY")
	soulTime:SetFontObject(GameFontHighlightSmall)
	soulTime:SetTextColor(1,1,1,1)
	soulTime:SetAllPoints()
	healthTime = self.buttons[5]:CreateFontString("DDHealthTime", "OVERLAY")
	healthTime:SetFontObject(GameFontHighlightSmall)
	healthTime:SetTextColor(1,1,1,1)
	healthTime:SetAllPoints()
	
	self.timerFrame = CreateFrame("Frame","DDTimerFrame")
	self.timerFrame:Hide()
	self.Timers = {}
	self.timerFrame:SetScript("OnUpdate", function(self, arg1) DemonDaemon.Timer(DemonDaemon, arg1) end)
	
	
	local timems = time()
	if self.db.profile.SoulTime or 0 > timems then
		self.Timers.SoulTimer = self.db.profile.SoulTime
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.HSTimer or 0 > timems then
		self.Timers.HSTimer = self.db.profile.HSTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.FelDomTimer or 0 > timems then
		self.Timers.FelDomTimer = self.db.profile.FelDomTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.ShadowWardTimer or 0 > timems then
		self.Timers.ShadowWardTimer = self.db.profile.ShadowWardTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.EmpowerTimer or 0 > timems then
		self.Timers.EmpowerTimer = self.db.profile.EmpowerTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.MetamorphTimer or 0 > timems then
		self.Timers.MetamorphTimer = self.db.profile.MetamorphTimer
		self.timerFrame:Show()
		self:Timer()
	end
	if self.db.profile.InfernalTimer or 0 > timems then
		self.Timers.InfernalTimer = self.db.profile.InfernalTimer
		self.timerFrame:Show()
		self:Timer()
	end
end

function DemonDaemon:UNIT_SPELLCAST_SUCCEEDED(event, unit, castGUID, spell)
	
	if unit == "player" then
	
		if self.spellID.UseSoulstone[spell] then
			self.Timers.SoulTimer = time() + 1800
			self.db.profile.SoulTime = self.Timers.SoulTimer
			self.timerFrame:Show()
			self:Timer()
		elseif self.spellID.UseHealthstone[spell] then
			self.Timers.HSTimer = time() + 120
			self.db.profile.HSTimer = self.Timers.HSTimer
			self.timerFrame:Show()
			self:Timer()
		elseif spell == 18708 then
			self.Timers.FelDomTimer = time() + 900
			self.db.profile.FelDomTimer = self.Timers.FelDomTimer
			self.timerFrame:Show()
			self:Timer()
		elseif spell == self.spellID.ShadowWard then
			self.Timers.ShadowWardTimer = time() + 30
			self.db.profile.ShadowWardTimer = self.Timers.ShadowWardTimer
			self.timerFrame:Show()
			self:Timer()
		--elseif spell == GetSpellInfo(self.spellID.Empower) then
		--	self.Timers.EmpowerTimer = time() + 60
		--	self.db.profile.EmpowerTimer = self.Timers.EmpowerTimer
		--	self.timerFrame:Show()
		--	self:Timer()
		
		--elseif spell == GetSpellInfo(self.spellID.Metamorph) then
		--	self.Timers.MetamorphTimer = time() + 180
		--	self.db.profile.MetamorphTimer = self.Timers.MetamorphTimer
		--	self.timerFrame:Show()
		--	self:Timer()
		
		elseif spell == self.spellID.Infernal then
			self.Timers.InfernalTimer = time() + 1200
			self.db.profile.InfernalTimer = self.Timers.InfernalTimer
			self.timerFrame:Show()
			self:Timer()
		end
		
	end
end

function DemonDaemon:Timer()
	if self.Timers.SoulTimer then
		local timeleft = self.Timers.SoulTimer - time()
		if(timeleft > 60)then
			soulTime:SetText((floor(timeleft / 60)) +1)
		else
			soulTime:SetText(timeleft)
		end
		soulTime:Show()
		
		if(timeleft < 1)then
			self.Timers.SoulTimer = nil
			soulTime:Hide()
			-- if self:getSoundState() then
				-- PlaySoundFile("Interface\\AddOns\\ShardAce\\Sounds\\Soulstone.mp3")
			-- end
			self.db.profile.SoulTime = nil
			DEFAULT_CHAT_FRAME:AddMessage("|cffffff00** Soulstone Cooldown over, ready to use **|r")
			UIErrorsFrame:AddMessage("** Soulstone Cooldown over, ready to use **", 0.92, 0.75, 0.05, 1.0, 12)
		end
	end
	if self.Timers.HSTimer then
		local timeleft = self.Timers.HSTimer - time()
		if(timeleft > 60)then
			healthTime:SetText((floor(timeleft / 60)) +1)
		else
			healthTime:SetText(timeleft)
		end
		healthTime:Show()
		
		if(timeleft < 1)then
			self.Timers.HSTimer = nil
			self.db.profile.HSTimer = nil
			healthTime:Hide()
		end
	end
	if self.Timers.FelDomTimer then
		local timeleft = self.Timers.FelDomTimer - time()
		if timeleft >= 1 then
			self.firstrow[1]:Desaturate(true)
		else
			self.firstrow[1]:Desaturate(false)
			self.Timers.FelDomTimer = nil
			self.db.profile.FelDomTimer = nil
		end
	end
	if self.Timers.ShadowWardTimer then
		local timeleft = self.Timers.ShadowWardTimer - time()
		if timeleft >= 1 then
			self.thirdrow[6]:Desaturate(true)
		else
			self.thirdrow[6]:Desaturate(false)
			self.Timers.ShadowWardTimer = nil
			self.db.profile.ShadowWardTimer = nil
		end
	end
	if self.Timers.EmpowerTimer then
		local timeleft = self.Timers.EmpowerTimer - time()
		if timeleft >= 1 then
			self.firstrow[5]:Desaturate(true)
		else
			self.firstrow[5]:Desaturate(false)
			self.Timers.EmpowerTimer = nil
			self.db.profile.EmpowerTimer = nil
		end
	end
	if self.Timers.MetamorphTimer then
		local timeleft = self.Timers.MetamorphTimer - time()
		if timeleft >= 1 then
			self.thirdrow[7]:Desaturate(true)
		else
			self.thirdrow[7]:Desaturate(false)
			self.Timers.MetamorphTimer = nil
			self.db.profile.MetamorphTimer = nil
		end
	end
	if self.Timers.InfernalTimer then
		local timeleft = self.Timers.InfernalTimer - time()
		if timeleft >= 1 then
			self.secondrow[5]:Desaturate(true)
		else
			self.secondrow[5]:Desaturate(false)
			self.Timers.InfernalTimer = nil
			self.db.profile.InfernalTimer = nil
		end
	end
		
		
	if not self.Timers.SoulTimer and not self.Timers.HSTimer and not self.Timers.FelDomTimer and not self.Timers.ShadowWardTimer and not self.Timers.EmpowerTimer and not self.Timers.MetamorphTimer then
		self.timerFrame:Hide()
	end
end

local shardCount
function DemonDaemon:BAG_UPDATE()
	if not shardCount then
		shardCount = self.buttons[1]:CreateFontString("DDShardCount", "OVERLAY")
		shardCount:SetFontObject(GameFontHighlightSmall)
		shardCount:SetTextColor(1,1,1,1)
		shardCount:SetAllPoints()
	else
		shardCount:SetText(GetItemCount(self.SoulShard))
	end
	local hsCount = 0
	for itemID,_ in pairs(self.HealthstoneID) do
		hsCount = hsCount + GetItemCount(itemID)
	end
	if hsCount == 0 then
		self.buttons[5]:Desaturate(true)
	else
		self.buttons[5]:Desaturate(false)
	end
	local ssCount = 0
	for itemID,_ in pairs(self.SoulstoneID) do
		ssCount = ssCount + GetItemCount(itemID)
	end
	if ssCount == 0 then
		self.buttons[2]:Desaturate(true)
	else
		self.buttons[2]:Desaturate(false)
	end
	local spCount = 0
	for itemID,_ in pairs(self.SpellstoneID) do
		spCount = spCount + GetItemCount(itemID)
	end
	if spCount == 0 then
		self.buttons[3]:Desaturate(true)
	else
		self.buttons[3]:Desaturate(false)
	end
	local fsCount = 0
	for itemID,_ in pairs(self.FirestoneID) do
		fsCount = fsCount + GetItemCount(itemID)
	end
	if fsCount == 0 then
		self.buttons[4]:Desaturate(true)
	else
		self.buttons[4]:Desaturate(false)
	end
end


function DemonDaemon:PLAYER_REGEN_ENABLED()
	if CreateButtonsFlag then
		self:CreateButtons()
		CreateButtonsFlag = nil
	end
	if SetupButtonsFlag then
		self:SetupButtons()
		SetupButtonsFlag = nil
	end
	if PosistionButtonsFlag then
		self:PosistionButtons()
		PosistionButtonsFlag = nil
	end
	
end

function DemonDaemon:SPELLS_CHANGED()
	self:SetupButtons()
	self:PosistionButtons()
end


function DemonDaemon:CreateButtons()
	
	if InCombatLockdown() then
		CreateButtonsFlag = true
		return
	end
	
	
	self.buttons = self.buttons or {
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", "Interface\\AddOns\\DemonDaemon\\blank", "SecureHandlerClickTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.CreateSS[1])), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.CreateSP[1])), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.CreateFS[1])), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.CreateHS[1])), "SecureActionButtonTemplate"),
	}
		
	self.firstrow = self.firstrow or {
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.FelDomination)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Enslave)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Sacrifice)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.SoulLink)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Empower)), "SecureActionButtonTemplate"),
	}	
	self.secondrow = self.secondrow or {
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Imp)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Voidwalker)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Succubus)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Felhunter)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Infernal)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Doomguard)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Felguard)), "SecureActionButtonTemplate"),
	}
	self.thirdrow = self.thirdrow or {
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.DetectInvis)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.UnendingBreath)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.SenseDemons)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.DemonSkin)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.FelArmor)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.ShadowWard)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Metamorph)), "SecureActionButtonTemplate"),
	}
	self.forthrow = self.forthrow or {
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Dreadsteed)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Summon)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Eye)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.Banish)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.SoulWell)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.SummonCircle)), "SecureActionButtonTemplate"),
		BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", select(3,GetSpellInfo(self.spellID.PortCircle)), "SecureActionButtonTemplate"),
	}
	
	self.anchor = self.anchor or BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", "Interface\\AddOns\\DemonDaemon\\blank", nil)
	self.anchor:Hide()
	 
	if strlower(self.db.profile.anchor or "") == "cursor" then
		self.anchor:SetPoint("CENTER",UIParent,"BOTTOMLEFT",self.db.profile.curx * UIParent:GetEffectiveScale(), self.db.profile.cury * UIParent:GetEffectiveScale() )
	else
		self.anchor:SetPoint("CENTER",UIParent,"BOTTOMLEFT",_G[self.db.profile.anchor or "Minimap"]:GetCenter() )
	end
	-- self.buttons[1]:RegisterForDrag("LeftButton")
	-- self.buttons[1]:SetScript("OnDragStart", function(self) self:StartMoving() end)
	-- self.buttons[1]:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
	self:BAG_UPDATE()
end
--LibStub("ButtonDaemon-1.0"):BD:NewButton("Interface\\AddOns\\DemonDaemon\\BtnRound", "Interface\\AddOns\\DemonDaemon\\blank", "SecureHandlerClickTemplate"):SetPoint("CENTER","PlayerPortrait")
--ArrangeCircle("PlayerPortrait", 35, 0, 360/6, {RuneButtonIndividual1, RuneButtonIndividual2,RuneButtonIndividual3,RuneButtonIndividual4,RuneButtonIndividual5,RuneButtonIndividual6})
function DemonDaemon:PosistionButtons()
	if InCombatLockdown() then
		PositionButtonsFlag = true
		return
	end

	for _, button in pairs(self.buttons) do
		button:ClearAllPoints()
		button:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT", 0,0)
		button:SetParent("UIParent")
		button:Show()
		button:Resize(26, 26)
		
	end

	for _, button in pairs(self.firstrow) do
		button:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT", 0,0)
		button:SetParent(self.buttons[1])
		button:Resize(26, 26)
	end
	for _, button in pairs(self.secondrow) do
		button:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT", 0,0)
		button:SetParent(self.buttons[1])
		button:Resize(26, 26)
	end
	for _, button in pairs(self.thirdrow) do
		button:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT", 0,0)
		button:SetParent(self.buttons[1])
		button:Resize(26, 26)
	end
	for _, button in pairs(self.forthrow) do
		button:SetPoint("BOTTOMRIGHT", UIParent, "TOPLEFT", 0,0)
		button:SetParent(self.buttons[1])
		button:Resize(26, 26)
	end
	
	if self:getStraightState() then
		local x,y = BD:GetXY(self:getRadius(), self:getAngle())
		BD:ArrangeStraight(self.anchor, x, y, self:getMainOffX(), self:getMainOffY(), self.buttons)
	else
		BD:ArrangeCircle(self.anchor, self:getRadius(), self:getAngle(), self:getOverrideState() and self:getIncrement() or (((math.deg(math.acos(13/self:getRadius())))-90)*2)* self.db.profile.reverse, self.buttons)
	end
	BD:ArrangeStraight(self.buttons[1], self:get1InitX(), self:get1InitY(), self:get1OffX(), self:get1OffY(), self.firstmap)
	BD:ArrangeStraight(self.buttons[1], self:get2InitX(), self:get2InitY(), self:get2OffX(), self:get2OffY(), self.secondmap)
	BD:ArrangeStraight(self.buttons[1], self:get3InitX(), self:get3InitY(), self:get3OffX(), self:get3OffY(), self.thirdmap)
	BD:ArrangeStraight(self.buttons[1], self:get4InitX(), self:get4InitY(), self:get4OffX(), self:get4OffY(), self.forthmap)
end


function DemonDaemon:SetupButtons()
	
	if InCombatLockdown() then
		SetupButtonsFlag = true
		return
	end
	
	
	local tmpspell, mount =0,0

	-- for i = 1, GetNumCompanions("MOUNT") do
		-- _,_,spellNum,_,_ = GetCompanionInfo("MOUNT", i)
		-- if spellNum == self.spellID.Felsteed then
			-- mount = spellNum
		-- elseif spellNum == self.spellID.Dreadsteed then
			-- mount = spellNum
			-- break
		-- end
	-- end
	--print(GetSpellInfo(GetSpellInfo(self.spellID.Felsteed)) or "no mount")


	
	
	local hsmacro = ""
	local ssmacro = ""
	local spmacro = ""
	local fsmacro = ""

	for id,_ in pairs(self.HealthstoneID) do
		hsmacro = format("/use item:%s\n%s", id, hsmacro)
	end
	for id,_ in pairs(self.SoulstoneID) do
		ssmacro = format("/use item:%s\n%s", id, ssmacro)
	end
	for id,_ in pairs(self.SpellstoneID) do
		spmacro = format("/use item:%s\n%s", id, spmacro)
	end
	for id,_ in pairs(self.FirestoneID) do
		fsmacro = format("/use item:%s\n%s", id, fsmacro)
	end

	

	self.buttons[1]:RegisterForClicks("AnyUp")
	--self.buttons[1]:Execute( [[MenuButtons = newtable(self:GetChildren())]])
	self.buttons[1]:SetAttribute("_onclick", [[
		
		if MenuButtons[1]:IsShown() then
			
			for i, button in ipairs(MenuButtons) do
				button:Hide()
			end
		else
			
			for i, button in ipairs(MenuButtons) do
				button:Show()
			end
		end
	
	]])

	self.buttons[2]:SetAttribute("type2", "spell")
	self.buttons[2]:SetAttribute("spell2", GetSpellInfo(GetSpellInfo(self.spellID.CreateSS[5])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateSS[4])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateSS[3])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateSS[2])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateSS[1])))

	self.buttons[3]:SetAttribute("type2", "spell")
	self.buttons[3]:SetAttribute("spell2", GetSpellInfo(GetSpellInfo(self.spellID.CreateSP[3])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateSP[2])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateSP[1])))
	

	self.buttons[4]:SetAttribute("type2", "spell")
	self.buttons[4]:SetAttribute("spell2", GetSpellInfo(GetSpellInfo(self.spellID.CreateFS[4])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateFS[3])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateFS[2])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateFS[1])))
	

	self.buttons[5]:SetAttribute("type2", "spell")
	self.buttons[5]:SetAttribute("spell2", GetSpellInfo(GetSpellInfo(self.spellID.CreateHS[5])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateHS[4])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateHS[3])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateHS[2])) or GetSpellInfo(GetSpellInfo(self.spellID.CreateHS[1])))
	

	self.buttons[2]:SetAttribute("type1", "macro")
	self.buttons[2]:SetAttribute("macro1", ATTRIBUTE_NOOP)
	self.buttons[2]:SetAttribute("macrotext1", ssmacro)

	self.buttons[3]:SetAttribute("type1", "macro")
	self.buttons[3]:SetAttribute("macro1", ATTRIBUTE_NOOP)
	self.buttons[3]:SetAttribute("macrotext1", spmacro)

	self.buttons[4]:SetAttribute("type1", "macro")
	self.buttons[4]:SetAttribute("macro1", ATTRIBUTE_NOOP)
	self.buttons[4]:SetAttribute("macrotext1", fsmacro)

	self.buttons[5]:SetAttribute("type1", "macro")
	self.buttons[5]:SetAttribute("macro1", ATTRIBUTE_NOOP)
	self.buttons[5]:SetAttribute("macrotext1", hsmacro)


	self.firstrow[1]:SetAttribute("type", "spell")
	self.firstrow[1]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.FelDomination)))
	self.firstrow[2]:SetAttribute("type", "spell")
	self.firstrow[2]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Enslave)))
	self.firstrow[3]:SetAttribute("type", "spell")
	self.firstrow[3]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Sacrifice)))
	self.firstrow[4]:SetAttribute("type", "spell")
	self.firstrow[4]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.SoulLink)))
	self.firstrow[5]:SetAttribute("type", "spell")
	self.firstrow[5]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Empower)))

	self.secondrow[1]:SetAttribute("type", "spell")
	self.secondrow[1]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Imp)))
	self.secondrow[2]:SetAttribute("type", "spell")
	self.secondrow[2]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Voidwalker)))
	self.secondrow[3]:SetAttribute("type", "spell")
	self.secondrow[3]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Succubus)))
	self.secondrow[4]:SetAttribute("type", "spell")
	self.secondrow[4]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Felhunter)))
	self.secondrow[5]:SetAttribute("type", "spell")
	self.secondrow[5]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Infernal)))
	self.secondrow[6]:SetAttribute("type", "spell")
	self.secondrow[6]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Doomguard)))
	self.secondrow[7]:SetAttribute("type", "spell")
	self.secondrow[7]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Felguard)))

	self.thirdrow[1]:SetAttribute("type", "spell")
	self.thirdrow[1]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.DetectInvis)))
	self.thirdrow[2]:SetAttribute("type", "spell")
	self.thirdrow[2]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.UnendingBreath)))
	self.thirdrow[3]:SetAttribute("type", "spell")
	self.thirdrow[3]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.SenseDemons)))
	tmpspell = GetSpellInfo(GetSpellInfo(self.spellID.DemonArmor))
	if tmpspell == nil then
		tmpspell = GetSpellInfo(GetSpellInfo(self.spellID.DemonSkin))
	end
	
	self.thirdrow[4]:SetAttribute("type", "spell")
	self.thirdrow[4]:SetAttribute("spell", tmpspell)
	self.thirdrow[5]:SetAttribute("type", "spell")
	self.thirdrow[5]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.FelArmor)))
	self.thirdrow[6]:SetAttribute("type", "spell")
	self.thirdrow[6]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.ShadowWard)))
	self.thirdrow[7]:SetAttribute("type", "spell")
	self.thirdrow[7]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Metamorph)))

	self.forthrow[1]:SetAttribute("type", "spell")
	self.forthrow[1]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Dreadsteed)) or GetSpellInfo(GetSpellInfo(self.spellID.Felsteed)))
	self.forthrow[2]:SetAttribute("type", "spell")
	self.forthrow[2]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Summon)))
	self.forthrow[3]:SetAttribute("type", "spell")
	self.forthrow[3]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Eye)))
	self.forthrow[4]:SetAttribute("type", "spell")
	self.forthrow[4]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.Banish)))
	self.forthrow[5]:SetAttribute("type", "spell")
	self.forthrow[5]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.SoulWell)))
	self.forthrow[5]:SetAttribute("type", "spell")
	self.forthrow[5]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.SummonCircle)))
	self.forthrow[5]:SetAttribute("type", "spell")
	self.forthrow[5]:SetAttribute("spell", GetSpellInfo(GetSpellInfo(self.spellID.PortCircle)))
	
	
	self.firstmap = {}
	self.secondmap = {}
	self.thirdmap = {}
	self.forthmap = {}



	for num, button in pairs(self.firstrow) do
		if button:GetAttribute("spell") then
			tinsert(self.firstmap, button)
		end
		button:Hide()
	end
	for num, button in pairs(self.secondrow) do
		if button:GetAttribute("spell") then
			tinsert(self.secondmap, button)
		end
		button:Hide()
	end
	for num, button in pairs(self.thirdrow) do
		if button:GetAttribute("spell") then
			tinsert(self.thirdmap, button)
		end
		button:Hide()
	end
	for num, button in pairs(self.forthrow) do
		if button:GetAttribute("spell") then
			tinsert(self.forthmap, button)
		end
		button:Hide()
	end

end
